@@ -53,10 +53,12 @@ sub do_macsuck {
         $device->ip;
       return;
   }
+  
+  my $ip = $device->ip;
 
   # would be possible just to use now() on updated records, but by using this
-  # same value for them all, we _can_ if we want add a job at the end to
-  # select and do something with the updated set (no reason to yet, though)
+  # same value for them all, we can if we want add a job at the end to
+  # select and do something with the updated set (see set archive, below)
   my $now = 'to_timestamp('. (join '.', gettimeofday) .')';
   my $total_nodes = 0;
 
@@ -70,13 +72,14 @@ sub do_macsuck {
   my $interfaces = $snmp->interfaces;
 
   # get forwarding table data via basic snmp connection
-  my $fwtable = { 0 => _walk_fwtable($device, $snmp, $interfaces, $port_macs, $device_ports) };
+  my $fwtable = _walk_fwtable($device, $snmp, $interfaces, $port_macs, $device_ports);
 
   # ...then per-vlan if supported
   my @vlan_list = _get_vlan_list($device, $snmp);
   foreach my $vlan (@vlan_list) {
       snmp_comm_reindex($snmp, $device, $vlan);
-      $fwtable->{$vlan} = _walk_fwtable($device, $snmp, $interfaces, $port_macs, $device_ports);
+      my $pv_fwtable = _walk_fwtable($device, $snmp, $interfaces, $port_macs, $device_ports, $vlan);
+      $fwtable = {%$fwtable, %$pv_fwtable};
   }
 
   # now it's time to call store_node for every node discovered
@@ -85,36 +88,45 @@ sub do_macsuck {
   # reverse sort allows vlan 0 entries to be included only as fallback
   foreach my $vlan (reverse sort keys %$fwtable) {
       foreach my $port (keys %{ $fwtable->{$vlan} }) {
-          if ($device_ports->{$port}->is_uplink) {
+          if ($device_ports->{$port}->is_uplink and not setting('macsuck_bleed')) {
               debug sprintf
                 ' [%s] macsuck - port %s is uplink, topo broken - skipping.',
-                $device->ip, $port;
+                $ip, $port;
               next;
           }
 
           debug sprintf ' [%s] macsuck - port %s vlan %s : %s nodes',
-            $device->ip, $port, $vlan, scalar keys %{ $fwtable->{$vlan}->{$port} };
+            $ip, $port, $vlan, scalar keys %{ $fwtable->{$vlan}->{$port} };
+
+          # make sure this port is UP in netdisco (unless it's a lag master,
+          # because we can still see nodes without a functioning aggregate)
+          $device_ports->{$port}->update({up_admin => 'up', up => 'up'})
+            if not $device_ports->{$port}->is_master;
 
           foreach my $mac (keys %{ $fwtable->{$vlan}->{$port} }) {
-              # make sure this port is UP in netdisco
-              $device_ports->{$port}->update({up_admin => 'up', up => 'up'});
-
-              # get VLAN from Q-BRIDGE if available
-              $vlan = $fwtable->{$vlan}->{$port}->{$mac}
-                if $vlan == 0;
 
               # remove vlan 0 entry for this MAC addr
               delete $fwtable->{0}->{$_}->{$mac}
                 for keys %{ $fwtable->{0} };
 
               ++$total_nodes;
-              store_node($device->ip, $vlan, $port, $mac, $now);
+              store_node($ip, $vlan, $port, $mac, $now);
           }
       }
   }
 
-  debug sprintf ' [%s] macsuck - %s forwarding table entries',
-    $device->ip, $total_nodes;
+  debug sprintf ' [%s] macsuck - %s updated forwarding table entries',
+    $ip, $total_nodes;
+
+  # a use for $now ... need to archive dissapeared nodes
+  my $archived = schema('netdisco')->resultset('Node')->search({
+    switch => $ip,
+    time_last => { '<' => \$now },
+  })->update({ active => \'false' });
+
+  debug sprintf ' [%s] macsuck - removed %d fwd table entries to archive',
+    $ip, $archived;
+
   $device->update({last_macsuck => \$now});
 }
 
@@ -140,48 +152,33 @@ sub store_node {
     my $nodes = schema('netdisco')->resultset('Node');
 
     # TODO: probably needs changing if we're to support VTP domains
-    my $old = $nodes->search({
-      mac => $mac,
-      vlan => $vlan,
-      -bool => 'active',
-      -not => {
-        switch => $ip,
-        port => $port,
-      },
-    });
-
-    # lock rows,
-    # and get the count so we know whether to set time_recent
-    my $old_count = scalar $old->search(undef,
-      {
-        columns => [qw/switch vlan port mac/],
-        for => 'update',
-      })->all;
-
-    $old->update({ active => \'false' });
-
-    my $new = $nodes->search({
-      'me.switch' => $ip,
-      'me.port' => $port,
-      'me.vlan' => $vlan,
-      'me.mac' => $mac,
-    });
+    my $old = $nodes->search(
+        { mac   => $mac,
+          -bool => 'active',
+          -not  => {
+                    switch => $ip,
+                    port   => $port,
+                    vlan   => $vlan,
+                  },
+        })->update( { active => \'false' } );
 
     # new data
-    $new->update_or_create(
+    $nodes->update_or_create(
       {
+        switch => $ip,
+        port => $port,
+        vlan => $vlan,
+        mac => $mac,
         active => \'true',
         oui => substr($mac,0,8),
         time_last => \$now,
-        ($old_count ? (time_recent => \$now) : ()),
+        (($old != 0) ? (time_recent => \$now) : ()),
       },
-      { for => 'update' }
+      {
+        key => 'primary',
+        for => 'update',
+      }
     );
-
-    # upgrade old schema
-    # an entry for this MAC existed in old schema with vlan => null
-    # which now has either vlan number or 0
-    $new->search({vlan => undef})->delete({only_nodes => 1});
   });
 }
 
@@ -299,7 +296,7 @@ sub _get_vlan_list {
 # walks the forwarding table (BRIDGE-MIB) for the device and returns a
 # table of node entries.
 sub _walk_fwtable {
-  my ($device, $snmp, $interfaces, $port_macs, $device_ports) = @_;
+  my ($device, $snmp, $interfaces, $port_macs, $device_ports, $comm_vlan) = @_;
   my $cache = {};
 
   my $fw_mac   = $snmp->fw_mac;
@@ -339,14 +336,6 @@ sub _walk_fwtable {
           next;
       }
 
-      # TODO: add proper port channel support!
-      if ($port =~ m/port.channel/i) {
-          debug sprintf
-            ' [%s] macsuck %s - port %s is LAG member - skipping.',
-            $device->ip, $mac, $port;
-          next;
-      }
-
       # this uses the cached $ports resultset to limit hits on the db
       my $device_port = $device_ports->{$port};
 
@@ -404,7 +393,15 @@ sub _walk_fwtable {
           next unless setting('macsuck_bleed');
       }
 
-      $cache->{$port}->{$mac} = ($fw_vlan->{$idx} || '0');
+      # possibly move node to lag master
+      if (defined $device_port->slave_of
+            and exists $device_ports->{$device_port->slave_of}) {
+          $port = $device_port->slave_of;
+          $device_ports->{$port}->update({is_uplink => \'true'});
+      }
+
+      my $vlan = $fw_vlan->{$idx} || $comm_vlan || '0';
+      ++$cache->{$vlan}->{$port}->{$mac};
   }
 
   return $cache;
