@@ -1,9 +1,13 @@
 package App::Netdisco::DB::ResultSet::Admin;
-use base 'DBIx::Class::ResultSet';
+use base 'App::Netdisco::DB::ResultSet';
 
 use strict;
 use warnings FATAL => 'all';
 
+__PACKAGE__->load_components(qw/
+  +App::Netdisco::DB::ExplicitLocking
+/);
+
 =head1 ADDITIONAL METHODS
 
 =head2 with_times
