@@ -4,6 +4,7 @@ use Dancer qw/:syntax :script/;
 use Dancer::Plugin::DBIC 'schema';
 
 use NetAddr::IP::Lite ':lower';
+use App::Netdisco::Util::DNS 'hostname_from_ip';
 
 use base 'Exporter';
 our @EXPORT = ();
@@ -59,10 +60,10 @@ sub get_device {
     ->find_or_new({ip => $ip});
 }
 
-=head2 check_acl( $ip, \@prefixes )
+=head2 check_acl( $ip, \@config )
 
-Given the IP address of a device, returns true if any of the IP prefixes in
-C<< \@prefixes >> contains that device, otherwise returns false.
+Given the IP address of a device, returns true if any of the items in C<<
+\@config >> matches that device, otherwise returns false.
 
 Normally you use C<check_no> and C<check_only>, passing the name of the
 configuration setting to load. This helper instead requires not the name of
@@ -76,6 +77,12 @@ sub check_acl {
   my $addr = NetAddr::IP::Lite->new($device->ip);
 
   foreach my $item (@$config) {
+      if (ref qr// eq ref $item) {
+          my $name = hostname_from_ip($addr->addr) or next;
+          return 1 if $name =~ $item;
+          next;
+      }
+
       if ($item =~ m/^([^:]+)\s*:\s*([^:]+)$/) {
           my $prop  = $1;
           my $match = $2;
@@ -84,14 +91,49 @@ sub check_acl {
           next unless $device->in_storage;
 
           # lazy version of vendor: and model:
-          if ($device->can($prop) and defined $device->prop
-              and $device->prop =~ m/^$match$/) {
+          if ($device->can($prop) and defined $device->$prop
+              and $device->$prop =~ m/^$match$/) {
               return 1;
           }
+
           next;
       }
 
-      my $ip = NetAddr::IP::Lite->new($item) or next;
+      if ($item =~ m/([a-f0-9]+)-([a-f0-9]+)$/i) {
+          my $first = $1;
+          my $last  = $2;
+
+          if ($item =~ m/:/) {
+              next unless $addr->bits == 128;
+
+              $first = hex $first;
+              $last  = hex $last;
+
+              (my $header = $item) =~ s/:[^:]+$/:/;
+              foreach my $part ($first .. $last) {
+                  my $ip = NetAddr::IP::Lite->new($header . sprintf('%x',$part) . '/128')
+                    or next;
+                  return 1 if $ip == $addr;
+              }
+          }
+          else {
+              next unless $addr->bits == 32;
+
+              (my $header = $item) =~ s/\.[^.]+$/./;
+              foreach my $part ($first .. $last) {
+                  my $ip = NetAddr::IP::Lite->new($header . $part . '/32')
+                    or next;
+                  return 1 if $ip == $addr;
+              }
+          }
+
+          next;
+      }
+
+      my $ip = NetAddr::IP::Lite->new($item)
+        or next;
+      next unless $ip->bits == $addr->bits;
+
       return 1 if $ip->contains($addr);
   }
 
@@ -116,6 +158,16 @@ Hostname, IP address, IP prefix
 
 =item *
 
+IP address range, using a hyphen and no whitespace
+
+=item *
+
+Regular Expression in YAML format which will match the device DNS name, e.g.:
+
+ - !!perl/regexp ^sep0.*$
+
+=item *
+
 C<"model:regex"> - matched against the device model
 
 =item *
@@ -157,6 +209,16 @@ Hostname, IP address, IP prefix
 
 =item *
 
+IP address range, using a hyphen and no whitespace
+
+=item *
+
+Regular Expression in YAML format which will match the device DNS name, e.g.:
+
+ - !!perl/regexp ^sep0.*$
+
+=item *
+
 C<"model:regex"> - matched against the device model
 
 =item *
