@@ -5,7 +5,7 @@ use Dancer::Plugin::DBIC;
 use Dancer::Plugin::Auth::Extensible;
 
 hook 'before' => sub {
-    params->{return_url} ||= '/inventory';
+    params->{return_url} ||= uri_for('/inventory');
 
     if (! session('logged_in_user') && request->path ne uri_for('/login')->path) {
         if (setting('trust_x_remote_user') and scalar request->header('X-REMOTE_USER')) {
